<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_Shipping_External_Fetch extends WC_Shipping_Method {

	public function __construct($instance_id = 0) {
		$this->id                 = 'external_fetch';
		$this->instance_id        = $instance_id;
		$this->method_title       = __( 'External Fetch' );
		$this->method_description = __( 'Description of your shipping method' );
		$this->supports           = array( 'zones', 'shipping-zones', 'instance-settings' );
		$this->enabled            = 'yes';
		$this->last_response = array();
	
		if ($instance_id == 0) {
			// If the constructor is called without $instance_id, there is nothing more to do.
			return;
		}
		
		$this->init();
	}
	
	
	function init() {
		$this->init_form_fields();
		$this->init_settings();
		
		$this->title              = $this->get_option( 'title', __( 'Label', 'woocommerce-external-fetch-shipping' ) );
		$this->description        = $this->get_option( 'description' );
		$this->method_description = $this->description;
		
		add_filter( 'woocommerce_cart_no_shipping_available_html', array( $this, 'filter_cart_no_shipping_available_html' ), 10, 1 );
		add_action( 'woocommerce_after_shipping_rate', array($this, 'action_after_shipping_rate'), 10, 2);
		add_action( 'woocommerce_proceed_to_checkout', array( $this, 'action_add_text_before_proceed_to_checkout' ));
		add_action( 'woocommerce_proceed_to_checkout', array( $this, 'maybe_clear_wc_shipping_rates_cache' ));
	}
	
	
	public function action_after_shipping_rate($rate, $index) {
		$rate_id = $rate->id;
		$rates = $this->last_response['rates'];
		foreach( $rates as $r ) {
			if ( $rate_id == $r['id'] ) { // This rate ID belongs to this instance
				echo "<div class='shipping_rate_description'>" . $r['description'] . "</div>";
			}
		}
	}
	
	public function maybe_clear_wc_shipping_rates_cache() {
		if ( $this->get_option('clear_wc_shipping_cache') == 'yes' ) {
			$packages = WC()->cart->get_shipping_packages();
			foreach ($packages as $key => $value) {
				$shipping_session = "shipping_for_package_$key";
				unset(WC()->session->$shipping_session);
			}
		}
	}
	
	
	public function action_add_text_before_proceed_to_checkout() {
		echo $this->last_response['before_checkout_button_html'];
	}
	
	
	public function filter_cart_no_shipping_available_html($previous) {
		return $previous . $this->last_response['cart_no_shipping_available_html'];
	}
	
	
	public function get_option( $key, $empty_value = null ) {
		// Instance options take priority over global options
		if ( in_array( $key, array_keys( $this->instance_form_fields ) ) ) {
			return $this->get_instance_option( $key, $empty_value );
		}

		// Return global option
		return parent::get_option( $key, $empty_value );
	}
	
	
	public function get_instance_option( $key, $empty_value = null ) {
		if ( empty( $this->instance_settings ) ) {
			$this->init_instance_settings();
		}

		// Get option default if unset.
		if ( ! isset( $this->instance_settings[ $key ] ) ) {
			$form_fields = $this->instance_form_fields;

			if ( is_callable( array( $this, 'get_field_default' ) ) ) {
				$this->instance_settings[ $key ] = $this->get_field_default( $form_fields[ $key ] );
			} else {
				$this->instance_settings[ $key ] = empty( $form_fields[ $key ]['default'] ) ? '' : $form_fields[ $key ]['default'];
			}
		}

		if ( ! is_null( $empty_value ) && '' === $this->instance_settings[ $key ] ) {
			$this->instance_settings[ $key ] = $empty_value;
		}

		return $this->instance_settings[ $key ];
	}
	
	
	public function get_instance_option_key() {
		return $this->instance_id ? $this->plugin_id . $this->id . '_' . $this->instance_id . '_settings' : '';
	}
	
	
	public function init_instance_settings() {
		// 2nd option is for BW compat
		$this->instance_settings = get_option( $this->get_instance_option_key(), get_option( $this->plugin_id . $this->id . '-' . $this->instance_id . '_settings', null ) );

		// If there are no settings defined, use defaults.
		if ( ! is_array( $this->instance_settings ) ) {
			$form_fields             = $this->get_instance_form_fields();
			$this->instance_settings = array_merge( array_fill_keys( array_keys( $form_fields ), '' ), wp_list_pluck( $form_fields, 'default' ) );
		}
	}
	
	
	public function init_form_fields() {
		$this->form_fields     = array(); // No global options for table rates
		$this->instance_form_fields = array(
			'title' => array(
				'title'       => __( 'Method Title', 'woocommerce-external-fetch-shipping' ),
				'type'        => 'text',
				'desc_tip'    => true,
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-external-fetch-shipping' ),
				'default'     => __( 'Fetch from localhost', 'woocommerce-external-fetch-shipping' )
			),
			'description' => array(
				'title'       => __( 'Method Description', 'woocommerce-external-fetch-shipping' ),
				'type'        => 'text',
				'desc_tip'    => true,
				'description' => __( 'This controls the description which the user sees during checkout.', 'woocommerce-external-fetch-shipping' ),
				'default'     => __( '', 'woocommerce-external-fetch-shipping' )
			),
			'endpoint' => array(
				'title'       => __( 'JSON API endpoint', 'woocommerce-external-fetch-shipping' ),
				'type'        => 'text',
				'desc_tip'    => true,
				'description' => __( 'Protocol, Hostname, Port and URL of API endpoint', 'woocommerce-external-fetch-shipping' ),
				'placeholder'     => __( 'http://localhost:4040/calculate', 'woocommerce-external-fetch-shipping' )
			),
			'fallback_cost' => array(
				'title'       => __( 'Fallback cost', 'woocommerce-external-fetch-shipping' ),
				'type'        => 'text',
				'desc_tip'    => true,
				'default'     => '0',
				'description' => __( 'Use this shipping cost when service unavailable', 'woocommerce-external-fetch-shipping' ),
			),
			'debug' => array(
				'title'       => __( 'Debug', 'woocommerce-external-fetch-shipping' ),
				'label'       => ' ',
				'type'        => 'checkbox',
				'default'     => 'no',
				'description' => __( 'Set a "debug": "yes" flag in the JSON sent to the service.', 'woocommerce-external-fetch-shipping' ),
			),
			'clear_wc_shipping_cache' => array(
				'title'       => __( 'Disable Shipping Cache', 'woocommerce-external-fetch-shipping' ),
				'label'       => ' ',
				'type'        => 'checkbox',
				'default'     => 'no',
				'description' => __( "Clear WooCommerce's session-based shipping calculation cache at every load.", 'woocommerce-external-fetch-shipping' ),
			),
		);
	}
	
	
	public function calculate_shipping( $package = array() ) {
		$endpoint = $this->get_option( 'endpoint' );
		if ( empty( $endpoint ) ) return;
	
		$request_data = [
			'destination' => [
				'country'   => $package['destination']['country'],
				'state'     => $package['destination']['state'],
				'postcode'  => $package['destination']['postcode'],
				'city'      => $package['destination']['city'],
				'address'   => $package['destination']['address'],
			],
			'items' => [],
			'debug' => $this->get_option('debug') === 'yes' ? true : false,
		];
	
		foreach ( $package['contents'] as $item ) {
			if ( ! $item['data']->needs_shipping() ) continue;
	
			$product = $item['data'];
			$request_data['items'][] = [
				'name'     => $product->get_name(),
				'quantity' => $item['quantity'],
				'weight'   => $product->get_weight(),
				'length'   => $product->get_length(),
				'width'    => $product->get_width(),
				'height'   => $product->get_height(),
			];
		}
	
		$response = wp_remote_post( $endpoint, [
			'method'    => 'POST',
			'body'      => json_encode($request_data),
			'headers'   => ['Content-Type' => 'application/json'],
			'timeout'   => 15,
		] );
	
		if ( is_wp_error( $response ) || empty( $response['body'] ) ) {
			// fallback rate
			$fallback = floatval( $this->get_option( 'fallback_cost', 0 ) );
			$this->add_rate([
				'id'    => $this->id,
				'label' => $this->title,
				'cost'  => $fallback,
			]);
			return;
		}
	
		$data = json_decode( wp_remote_retrieve_body( $response ), true );
	
		if ( isset( $data['rates'] ) && is_array( $data['rates'] ) ) {
			$this->last_response['rates'] = $data['rates'];
	
			foreach ( $data['rates'] as $rate ) {
				$this->add_rate([
					'id'       => $rate['id'],
					'label'    => $rate['label'],
					'cost'     => floatval( $rate['cost'] ),
					'package'  => $package,
				]);
			}
		}
	}
	
	
	public function add_free_shipping( $reason = '', $package = array() ) {
		$fallback_cost = $this->get_option('fallback_cost');
		$rate = array(
			'id' => $this->id,
			'label' => "E" . $reason,
			'cost' => $fallback_cost
		);
		$this->add_rate( $rate, $package );
	}
}
